import pandas as pd
import json

class Quizgen:
    def __init__(self):
        self.db = pd.read_json("pyquizrd/jeopardy/pruned_jeopardy.json")
        self.db.category = self.db.category.str.title().str.strip()

    def __str__(self):
        return "jeopardy quiz generator"

    def get_topics(self, num=100):
        return sorted(self.db.category.value_counts()[:num].index.tolist())

    def get_topic_formats(self):
        return ["multiple-choice"]

    def get_answer_formats(self):
        return ["freeform"]

    def gen_quiz(self, topic, num_questions, num_answers=1, difficulty=3, temperature=None):
        if topic not in self.db.category.unique():
            raise Exception(f"unknown topic {topic}")
        filtered = self.db.loc[self.db["category"] == topic]
        round1 = "Jeopardy!"
        round2 = "Double Jeopardy!"
        round3 = "Final Jeopardy!"
        value1 = f"${difficulty * 100}"
        value2 = f"${difficulty * 200}"
        filtered = filtered.loc[
            (filtered["round"] == round1) & (filtered["value"] == value1) |
            (filtered["round"] == round2) & (filtered["value"] == value2) |
            (filtered["round"] == round3) & (difficulty == 5)
        ]
        filtered = filtered[["question", "answer"]]
        filtered = filtered.rename(columns={"answer": "correct"})
        filtered = filtered.sample(num_questions)
        if num_questions < len(filtered.index):
            filtered = filtered.sample(n=num_questions)
        return filtered.to_json(orient="records", indent=4)



import glob
import json
import requests
import random

# https://opentdb.com/

class Quizgen:
    def __init__(self):
        self.topics = ("General Knowledge", "Books", "Film", "Music",
            "Musicals & Theatres", "Television", "Video Games", "Board Games",
            "Science & Nature", "Computers", "Mathematics", "Mythology",
            "Sports", "Geography", "History", "Politics", "Art", "Celebrities",
            "Animals", "Vehicles", "Comics", "Gadgets", "Japanese Anime & Manga",
            "Cartoons &  Animations")

    def __str__(self):
        return "opentrivia quiz generator"

    def get_topics(self, num=None):
        return self.topics

    def get_topic_formats(self):
        return ["multiple-choice"]

    def get_answer_formats(self):
        return ["multiple-choice", "true/false"]

    def gen_quiz(self, topic, num_questions, num_answers, difficulty=3, temperature=None):
        if difficulty <= 2:
            difficulty_word = "easy"
        elif difficulty <= 4:
            difficulty_word = "medium"
        elif difficulty == 5:
            difficulty_word = "hard"
        topic_num = self.topics.index(topic) + 9
       
        url = "https://opentdb.com/api.php?"
        url += f"amount={num_questions}"
        url += f"&category={topic_num}"
        url += f"&difficulty={difficulty_word}"
        url += f"&type=multiple"

        r = requests.get(url)
        quiz = r.json()["results"]
        json_quiz = []
        
        for question in quiz:
            json_quiz.append({
                "question": question["question"],
                "correct": question["correct_answer"],
                "responses": [question["correct_answer"]] + question["incorrect_answers"]
            })
        # randomize responses
        for i in json_quiz:
            random.shuffle(i["responses"])
        quiz = json.dumps(json_quiz, indent=4)
        return quiz




import json
import pprint
import google.generativeai as palm
import importlib
import os
import random
import vertexai
from vertexai.preview.language_models import TextGenerationModel

class Quizgen:
    def __init__(self, project="quizrd-prod-382117", location="us-central1"):
        self.topics = set()
        vertexai.init(project=project, location=location)
        data_path = os.path.join(os.path.dirname(__file__), "prompt.txt")
        with open(data_path, encoding='utf-8') as fp:
            self.prompt = fp.read()

    def __str__(self):
        return "palm quiz generator"

    def get_topics(self, num=None):
        return self.topics

    def get_topic_formats(self):
        return ["free-form"]

    def get_answer_formats(self):
        return ["free-form", "multiple-choice"]

    def predict_llm(self, model, temp, tokens, top_p, top_k, content, tuned_model=""):
      m = TextGenerationModel.from_pretrained(model)
      if tuned_model:
          m = model.get_tuned_model(tuned_model)
      response = m.predict(content, temperature=temp, max_output_tokens=tokens,
          top_k=top_k, top_p=top_p)
      return response.text

    def gen_quiz(self, topic, num_questions, num_answers, difficulty=3, temperature=.5):
        if difficulty <= 2:
            difficulty_word = "easy"
        elif difficulty <= 4:
            difficulty_word = "medium"
        elif difficulty <= 5:
            difficulty_word = "difficult"

        prompt = self.prompt.format(topic=topic,
            num_questions=num_questions,
            num_answers=num_answers,
            difficulty=difficulty)
        quiz = self.predict_llm("text-bison@001", temperature, 1024, 0.8, 40, prompt)
        # randomize responses
        json_quiz = json.loads(quiz)
        for i in json_quiz:
            random.shuffle(i["responses"])
        quiz = json.dumps(json_quiz, indent=4)
        return quiz



import pandas as pd
import json

class Quizgen:
    def __init__(self):
        self.topics = set()

    def __str__(self):
        return "manual quiz generator"

    def get_topics(self, num=100):
        return self.topics

    def get_topic_formats(self):
        return ["freeform"]

    def get_answer_formats(self):
        return ["freeform", "multiple-choice"]

    def gen_quiz(self, topic, num_questions, num_answers, difficulty, temperature=None):
        return "[]"



import importlib
import pyquizrd
import glob
import os

GENERATORS = {
    "gpt":        None,
    "jeopardy":   None,
    "manual":     None,
    "opentrivia": None,
    "palm":       None
}

class Quizgen:
    def __init__(self, type):
        if type not in GENERATORS:
            raise Exception(f"Unsupported generator type {type}.")
        if not GENERATORS[type]:
            mod = importlib.import_module(".." + type + "." + type, package="pyquizrd.pyquizrd")
            GENERATORS[type] = mod
        self.type = type
        self._gen = GENERATORS[type].Quizgen()

    def __str__(self):
        return self._gen.__str__()

    def get_gens():
        gens = {}
        for g in GENERATORS:
           gens[g] = {
               "topic_formats":  Quizgen(g).get_topic_formats(),
               "answer_formats": Quizgen(g).get_answer_formats(),
               "topics":         Quizgen(g).get_topics()
           }
        return gens

    def get_mode(self):
        return self._gen.get_mode()
    
    def get_topics(self, num=100):
        return self._gen.get_topics(num)

    def get_topic_formats(self):
        return self._gen.get_topic_formats()

    def get_answer_formats(self):
        return self._gen.get_answer_formats()

    def gen_quiz(self, topic, num_questions, num_answers, difficulty, temperature):
        return self._gen.gen_quiz(topic, num_questions, num_answers, difficulty, temperature)
